@@ -50,13 +50,11 @@ class SmartFacade(object):
     def _reset(self):
         # This attribute is initialized lazily in the _get_ctrl() method.
         self._ctrl = None
-
         self._pkg2hash = {}
         self._hash2pkg = {}
-
         self._marks = {}
-
         self._caching = ALWAYS
+        self._channels_reloaded = False
 
     def deinit(self):
         """Deinitialize the Facade and the Smart library."""
@@ -87,6 +85,13 @@ class SmartFacade(object):
     def smart_initialized(self):
         """Hook called when the Smart library is initialized."""
 
+    def ensure_channels_reloaded(self):
+        """Reload the channels if they haven't been reloaded yet."""
+        if self._channels_reloaded:
+            return
+        self._channels_reloaded = True
+        self.reload_channels()
+
     def reload_channels(self):
         """
         Reload Smart channels, getting all the cache (packages) in memory.
@@ -351,9 +356,14 @@ class SmartFacade(object):
         self._get_ctrl()
         smart.pkgconf.setFlag("lock", name, relation, version)
 
-    def remove_package_lock(self, name, relation=(), version=()):
+    def remove_package_lock(self, name, relation=None, version=None):
         """Remove a package lock."""
         self._validate_lock_condition(relation, version)
         self._get_ctrl()
         smart.pkgconf.clearFlag("lock", name=name, relation=relation,
                                 version=version)
+
+    def save_config(self):
+        """Flush the current smart configuration to disk."""
+        control = self._get_ctrl()
+        control.saveSysConf()
