@@ -12,6 +12,10 @@ from landscape.package.store import PackageStore, InvalidHashIdDb
 from landscape.broker.remote import RemoteBroker
 
 
+class PackageTaskError(Exception):
+    """Raised when a task hasn't been successfully completed."""
+
+
 class PackageTaskHandlerConfiguration(Configuration):
     """Specialized configuration for L{PackageTaskHandler}s."""
 
@@ -66,39 +70,67 @@ class PackageTaskHandler(object):
         self._facade = package_facade
         self._broker = remote_broker
         self._config = config
-        self._channels_reloaded = False
-
-    def ensure_channels_reloaded(self):
-        if not self._channels_reloaded:
-            self._channels_reloaded = True
-            self._facade.reload_channels()
+        self._count = 0
 
     def run(self):
         return self.handle_tasks()
 
     def handle_tasks(self):
+        """Handle the tasks in the queue.
+
+        The tasks will be handed over one by one to L{handle_task} until the
+        queue is empty or a task fails.
+
+        @see: L{handle_tasks}
+        """
         return self._handle_next_task(None)
 
     def _handle_next_task(self, result, last_task=None):
+        """Pick the next task from the queue and pass it to C{handle_task}."""
+
         if last_task is not None:
             # Last task succeeded.  We can safely kill it now.
             last_task.remove()
+            self._count += 1
 
         task = self._store.get_next_task(self.queue_name)
 
         if task:
             # We have another task.  Let's handle it.
             result = self.handle_task(task)
-            result.addCallback(self._handle_next_task, task)
+            result.addCallback(self._handle_next_task, last_task=task)
+            result.addErrback(self._handle_task_failure)
             return result
 
         else:
             # No more tasks!  We're done!
             return succeed(None)
 
+    def _handle_task_failure(self, failure):
+        """Gracefully handle a L{PackageTaskError} and stop handling tasks."""
+        failure.trap(PackageTaskError)
+
     def handle_task(self, task):
+        """Handle a sigle task.
+
+        Sub-classes must override this method in order to trigger task-specific
+        actions.
+
+        This method must return a L{Deferred} firing the task result. If the
+        deferred is successful the task will be removed from the queue and the
+        next one will be picked. If the task can't be completed, this method
+        must raise a L{PackageTaskError}, in this case the handler will stop
+        processing tasks and the failed task won't be removed from the queue.
+        """
         return succeed(None)
 
+    @property
+    def handled_tasks_count(self):
+        """
+        Return the number of tasks that have been successfully handled so far.
+        """
+        return self._count
+
     def use_hash_id_db(self):
         """
         Attach the appropriate pre-canned hash=>id database to our store.
