@@ -36,16 +36,6 @@ class PackageTaskHandlerTest(LandscapeIsolatedTest):
         self.handler = PackageTaskHandler(self.store, self.facade, self.remote,
                                           self.config)
 
-    def test_ensure_channels_reloaded(self):
-        self.assertEquals(len(self.facade.get_packages()), 0)
-        self.handler.ensure_channels_reloaded()
-        self.assertEquals(len(self.facade.get_packages()), 3)
-
-        # Calling it once more won't reload channels again.
-        self.facade.get_packages_by_name("name1")[0].installed = True
-        self.handler.ensure_channels_reloaded()
-        self.assertTrue(self.facade.get_packages_by_name("name1")[0].installed)
-
     def test_use_hash_id_db(self):
 
         # We don't have this hash=>id mapping
@@ -467,7 +457,8 @@ class PackageTaskHandlerTest(LandscapeIsolatedTest):
         done = Deferred()
         self.expect(reactor_mock.stop()).call(lambda: done.callback(None))
 
-        class MyException(Exception): pass
+        class MyException(Exception):
+            pass
 
         self.log_helper.ignore_errors(MyException)
 
@@ -480,10 +471,9 @@ class PackageTaskHandlerTest(LandscapeIsolatedTest):
 
         # Ok now for some real stuff
 
-        result = run_task_handler(handler_factory_mock,
-                                  ["--data-path", self.data_path,
-                                   "--bus", "session"],
-                                  reactor=reactor_mock)
+        run_task_handler(handler_factory_mock,
+                         ["--data-path", self.data_path, "--bus", "session"],
+                         reactor=reactor_mock)
 
         def everything_stopped(result):
             self.assertIn("MyException", self.logfile.getvalue())
@@ -493,7 +483,6 @@ class PackageTaskHandlerTest(LandscapeIsolatedTest):
 
 class LazyRemoteBrokerTest(LandscapeTest):
 
-
     def test_wb_is_lazy(self):
         """
         The L{LazyRemoteBroker} class doesn't initialize the actual remote
