@@ -455,6 +455,9 @@ class LandscapeSetupScriptTest(LandscapeTest):
                           "FooPlugin, ScriptExecution")
 
     def test_query_script_users_defined_on_command_line(self):
+        """
+        Confirm with the user for users specified for the ScriptPlugin.
+        """
         self.config.include_manager_plugins = "FooPlugin"
         self.mocker.order()
         script_mock = self.mocker.patch(self.script)
@@ -462,8 +465,8 @@ class LandscapeSetupScriptTest(LandscapeTest):
         script_mock.prompt_yes_no("Enable script execution?", default=False)
         self.mocker.result(True)
         script_mock.show_help(ANY)
-        raw_input_mock = self.mocker.replace(raw_input, passthrough=False)
-        self.expect(raw_input_mock(ANY)).count(0)
+        script_mock.prompt_get_input(
+            "Script users [root, nobody, landscape]: ", False)
         self.mocker.replay()
 
         self.config.load_command_line(
@@ -472,6 +475,128 @@ class LandscapeSetupScriptTest(LandscapeTest):
         self.assertEquals(self.config.script_users,
                           "root, nobody, landscape")
 
+    def test_query_script_users_defined_on_command_line_with_unknown_user(self):
+        """
+        If several users are provided on the command line, we verify the users
+        and raise a ConfigurationError if any are unknown on this system.
+        """
+        pwnam_mock = self.mocker.replace("pwd.getpwnam")
+        pwnam_mock("root")
+        self.mocker.result(None)
+        pwnam_mock("nobody")
+        self.mocker.result(None)
+        pwnam_mock("landscape")
+        self.mocker.result(None)
+        pwnam_mock("unknown")
+        self.mocker.throw(KeyError())
+        self.mocker.replay()
+
+        self.config.load_command_line(
+            ["--script-users", "root, nobody, landscape, unknown",
+            "--include-manager-plugins", "ScriptPlugin"])
+        self.assertRaises(ConfigurationError, self.script.query_script_plugin)
+
+    def test_query_script_users_defined_on_command_line_with_all_user(self):
+        """
+        We shouldn't accept all as a synonym for ALL
+        """
+        self.config.load_command_line(
+            ["--script-users", "all",
+            "--include-manager-plugins", "ScriptPlugin"])
+        self.assertRaises(ConfigurationError, self.script.query_script_plugin)
+
+    def test_query_script_users_defined_on_command_line_with_ALL_user(self):
+        """
+        ALL is the special marker for all users.
+        """
+        self.config.load_command_line(
+            ["--script-users", "ALL",
+             "--include-manager-plugins", "ScriptPlugin"])
+        self.script.query_script_plugin()
+        self.assertEquals(self.config.script_users,
+                          "ALL")
+
+    def test_query_script_users_defined_on_command_line_with_ALL_and_extra_user(self):
+        """
+        If ALL and additional users are provided as the users on the command
+        line, this should raise an appropriate ConfigurationError.
+        """
+        self.config.load_command_line(
+            ["--script-users", "ALL, kevin",
+            "--include-manager-plugins", "ScriptPlugin"])
+        self.assertRaises(ConfigurationError, self.script.query_script_plugin)
+
+    def test_invalid_user_entered_by_user(self):
+        """
+        If an invalid user is entered on the command line the user should be
+        informed and prompted again.
+        """
+        help_snippet = "Landscape has a feature which enables administrators"
+        self.mocker.order()
+        script_mock = self.mocker.patch(self.script)
+        script_mock.show_help(self.get_matcher(help_snippet))
+        script_mock.prompt_yes_no("Enable script execution?", default=False)
+        self.mocker.result(True)
+        script_mock.show_help(
+            self.get_matcher("By default, scripts are restricted"))
+        script_mock.prompt_get_input("Script users: ", False)
+        self.mocker.result(u"nonexistent")
+        script_mock.show_help("Unknown system users: nonexistent")
+        script_mock.prompt_get_input("Script users: ", False)
+        self.mocker.result(u"root")
+        self.mocker.replay()
+        self.script.query_script_plugin()
+        self.assertEquals(self.config.script_users,
+                          "root")
+
+    def test_tags_not_defined_on_command_line(self):
+        """
+        If tags are not provided, the user should be prompted for them.
+        """
+        self.mocker.order()
+        script_mock = self.mocker.patch(self.script)
+        script_mock.show_help("You may provide tags for this computer e.g. "
+                              "server,hardy.")
+        script_mock.prompt("tags", "Tags", False)
+        self.mocker.replay()
+        self.script.query_tags()
+
+    def test_invalid_tags_entered_by_user(self):
+        """
+        If tags are not provided, the user should be prompted for them, and
+        they should be valid tags, if not the user should be prompted for them
+        again.
+        """
+        script_mock = self.mocker.patch(self.script)
+        script_mock.show_help("You may provide tags for this computer e.g. "
+                              "server,hardy.")
+        script_mock.prompt_get_input("Tags: ", False)
+        self.mocker.result(u"<script>alert();</script>")
+        script_mock.show_help("Tag names may only contain alphanumeric "
+                              "characters.")
+        script_mock.prompt_get_input("Tags: ", False)
+        self.mocker.result(u"london")
+        self.mocker.replay()
+        self.script.query_tags()
+
+    def test_tags_defined_on_command_line(self):
+        """
+        Tags defined on the command line can be verified by the user.
+        """
+        raw_input_mock = self.mocker.replace(raw_input, passthrough=False)
+        self.expect(raw_input_mock(ANY)).count(0)
+        self.mocker.replay()
+        self.config.load_command_line(["--tags", u"server,london"])
+        self.script.query_tags()
+        self.assertEquals(self.config.tags, u"server,london")
+
+    def test_invalid_tags_defined_on_command_line_raises_error(self):
+        raw_input_mock = self.mocker.replace(raw_input, passthrough=False)
+        self.expect(raw_input_mock(ANY)).count(0)
+        self.mocker.replay()
+        self.config.load_command_line(["--tags", u"<script>alert();</script>"])
+        self.assertRaises(ConfigurationError, self.script.query_tags)
+
     def test_show_header(self):
         help_snippet = "This script will"
         script_mock = self.mocker.patch(self.script)
@@ -488,6 +613,7 @@ class LandscapeSetupScriptTest(LandscapeTest):
         script_mock.query_registration_password()
         script_mock.query_proxies()
         script_mock.query_script_plugin()
+        script_mock.query_tags()
         self.mocker.replay()
 
         self.script.run()
@@ -525,7 +651,8 @@ class ConfigurationFunctionsTest(LandscapeTest):
                                  "http_proxy = http://old.proxy\n"
                                  "https_proxy = https://old.proxy\n"
                                  "url = http://url\n"
-                                 "include_manager_plugins = ScriptExecution"
+                                 "include_manager_plugins = ScriptExecution\n"
+                                 "tags = london, server"
                                  )
 
         raw_input = self.mocker.replace("__builtin__.raw_input",
@@ -541,6 +668,8 @@ class ConfigurationFunctionsTest(LandscapeTest):
         expect(raw_input(C("[http://old.proxy]"))).result("http://new.proxy")
         expect(raw_input(C("[https://old.proxy]"))).result("https://new.proxy")
         expect(raw_input(C("Enable script execution? [Y/n]"))).result("n")
+        expect(raw_input(C("Tags [london, server]: "))).result(
+            u"glasgow, laptop")
 
         # Negative assertion.  We don't want it called in any other way.
         expect(raw_input(ANY)).count(0)
@@ -550,7 +679,6 @@ class ConfigurationFunctionsTest(LandscapeTest):
         expect(print_text_mock(ANY)).count(0, None)
 
         self.mocker.replay()
-
         config = self.get_config(["--no-start", "--config", filename])
         setup(config)
         self.assertEquals(type(config), LandscapeSetupConfiguration)
@@ -564,6 +692,7 @@ class ConfigurationFunctionsTest(LandscapeTest):
         self.assertEquals(config.http_proxy, "http://new.proxy")
         self.assertEquals(config.https_proxy, "https://new.proxy")
         self.assertEquals(config.include_manager_plugins, "")
+        self.assertEquals(config.tags, u"glasgow, laptop")
 
     def test_silent_setup(self):
         """
@@ -637,6 +766,23 @@ script_users = root, nobody
 account_name = account
 """)
 
+    def test_silent_script_users_with_all_user(self):
+        """
+        In silent mode, we shouldn't accept invalid users, it should raise a
+        configuration error.
+        """
+        sysvconfig_mock = self.mocker.patch(SysVConfig)
+        sysvconfig_mock.set_start_on_boot(True)
+        self.mocker.replay()
+
+        config = self.get_config(
+            ["--script-users", "all",
+             "--include-manager-plugins", "ScriptPlugin",
+             "-a", "account",
+             "-t", "rex",
+             "--silent"])
+        self.assertRaises(ConfigurationError, setup, config)
+
     def test_silent_setup_with_ping_url(self):
         sysvconfig_mock = self.mocker.patch(SysVConfig)
         sysvconfig_mock.set_start_on_boot(True)
@@ -806,7 +952,7 @@ account_name = account
         sysvconfig_mock.set_start_on_boot(True)
         sysvconfig_mock.restart_landscape()
         self.mocker.throw(ProcessError)
-        
+
         print_text_mock("Couldn't restart the Landscape client.", error=True)
         print_text_mock(CONTAINS("This machine will be registered"), error=True)
 
@@ -827,7 +973,7 @@ account_name = account
         sysvconfig_mock.set_start_on_boot(True)
         sysvconfig_mock.restart_landscape()
         self.mocker.throw(ProcessError)
-        
+
         print_text_mock("Couldn't restart the Landscape client.", error=True)
         print_text_mock(CONTAINS("This machine will be registered"), error=True)
 
@@ -1169,7 +1315,7 @@ account_name = account
             self.get_config(["--config", config_filename, "--silent",
                              "--import", "https://config.url"])
         except ImportOptionError, error:
-            self.assertEquals(str(error), 
+            self.assertEquals(str(error),
                               "Couldn't download configuration from "
                               "https://config.url: Server "
                               "returned HTTP code 501")
@@ -1192,7 +1338,7 @@ account_name = account
             self.get_config(["--config", config_filename, "--silent",
                              "--import", "https://config.url"])
         except ImportOptionError, error:
-            self.assertEquals(str(error), 
+            self.assertEquals(str(error),
                               "Couldn't download configuration from "
                               "https://config.url: Error 60: pycurl message")
         else:
@@ -1261,7 +1407,7 @@ account_name = account
         self.addCleanup(os.remove, key_filename)
 
         print_text_mock = self.mocker.replace(print_text)
-        print_text_mock("Writing SSL public key to %s..." % key_filename)
+        print_text_mock("Writing SSL CA certificate to %s..." % key_filename)
 
         self.mocker.replay()
 
