@@ -2,7 +2,7 @@ from landscape.lib import md5crypt
 from landscape.user.management import UserManagement, UserManagementError
 from landscape.user.tests.helpers import FakeUserProvider
 from landscape.user.provider import UserNotFoundError, GroupNotFoundError
-from landscape.tests.helpers import LandscapeTest, MakePathHelper, MockPopen
+from landscape.tests.helpers import LandscapeTest, MockPopen
 
 
 def guess_password(generated_password, plaintext_password):
@@ -13,11 +13,9 @@ def guess_password(generated_password, plaintext_password):
 
 class UserWriteTest(LandscapeTest):
 
-    helpers = [MakePathHelper]
-
     def setUp(self):
         LandscapeTest.setUp(self)
-        self.shadow_file = self.make_path("""\
+        self.shadow_file = self.makeFile("""\
 jdoe:$1$xFlQvTqe$cBtrNEDOIKMy/BuJoUdeG0:13348:0:99999:7:::
 psmith:!:13348:0:99999:7:::
 sbarnes:$1$q7sz09uw$q.A3526M/SHu8vUb.Jo1A/:13349:0:99999:7:::
@@ -380,11 +378,9 @@ sbarnes:$1$q7sz09uw$q.A3526M/SHu8vUb.Jo1A/:13349:0:99999:7:::
 
 class GroupWriteTest(LandscapeTest):
 
-    helpers = [MakePathHelper]
-
     def setUp(self):
         LandscapeTest.setUp(self)
-        self.shadow_file = self.make_path("""\
+        self.shadow_file = self.makeFile("""\
 jdoe:$1$xFlQvTqe$cBtrNEDOIKMy/BuJoUdeG0:13348:0:99999:7:::
 psmith:!:13348:0:99999:7:::
 sbarnes:$1$q7sz09uw$q.A3526M/SHu8vUb.Jo1A/:13349:0:99999:7:::
